/* $LAAS: viamPom.c 2008/08/04 16:33:15 mallet $ */

/*
 * Copyright (c) 2007-2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Mon Mar  5 2007
 */

#include <portLib.h>
#include <stdio.h>
#include <err.h>

#include <viam/viamlib.h>

#include "server/viamHeader.h"
#include "shared.h"


/* --- local data ---------------------------------------------------- */

#define POM_FRAME_NAME      "pom%sFramePos"

const POM_SENSOR_POS viam_poszero = {
  .date = 0,
  .sensorToMain = {
    .euler = { 0. /*all zeros*/ },
    .var = { 0. /*all zeros*/ },
  },
  .mainToBase = {
     .euler = { 0. /*all zeros*/ },
     .var = { 0. /*all zeros*/ },
   },
  .mainToOrigin = {
     .euler = { 0. /*all zeros*/ },
     .var = { 0. /*all zeros*/ },
   },
  .VLocal = {
     .euler = { 0. /*all zeros*/ },
     .var = { 0. /*all zeros*/ },
   }
};


/* --- viamPomInit --------------------------------------------------- */

/** Make a new connection to pom.
 */

STATUS
viamPomInit(viam_bank_t bank, int *report)
{
  viam_bankdata_t *d = viam_bank_usrdata(bank);
  char posterName[256];
  int i;

  if (!d) return ERROR;

  for(i=0; i<viam_bank_nimages(bank); i++) {
    viam_image_t c = viam_bank_image(vh, bank, i);
    viam_imagedata_t *idata = viamIData(bank, i);
    if (!c || !idata) continue;

    idata->pomFrameId = NULL;
    if (d->tags != VIAM_ENABLE) continue;

#if 0
    /* XXX wait for simulation mode to be implemented */
    if (SDI_F->simuMode) continue;
#endif
    sprintf(posterName, POM_FRAME_NAME, viam_image_name(c));
    if (posterFind(posterName, &idata->pomFrameId) != OK) {
      warnx("no such pom frame: `%s'", posterName);
      *report = S_viam_NO_SUCH_POM_FRAME;
      return ERROR;
    }
  }

  return OK;
}


/* --- viamPomGetTags ------------------------------------------------ */

/** Read the frame position in pom poster */

int
viamPomGetTags(viam_bank_t bank)
{
  viam_bankdata_t *d = viam_bank_usrdata(bank);
  viam_imagedata_t *idata;
  STATUS s;
  int i;

  if (!d) return S_viam_NOT_CONFIGURED;
  if (d->tags != VIAM_ENABLE) return 0;

  for(i=0; i<viam_bank_nimages(bank); i++) {
    idata = viamIData(bank, i);
    if (!idata || idata->pomFrameId == NULL) continue;

#if 0
    /* XXX wait for simulation mode to be implemented */
    if (SDI_F->simuMode) {
      for(i=0;i<b->input->images;i++) {
	sprintf(SDI_F->fileName, "%s/" CAMERA_SAVE_TAG_NAME "%03d%s",
		SDI_F->simP.directory, SDI_F->simP.number,
		b->input->image[i].fileExt);

	if (pomReadSensorPos(SDI_F->fileName, &b->pomTag[i])) {
	  camera_error("cameraPomTag", bilan, CANNOT_TAG_IMAGE);
	  return ERROR;
	}
      }
    }
#endif

    s = viamPOM_SENSOR_POSPosterRead(idata->pomFrameId, &idata->pomFrame);
    if (s != OK) {
      warnx("unreadable pom frame: `%s'",
	    viam_image_name(viam_bank_image(vh, bank, i)));
      return S_viam_NO_SUCH_POM_FRAME;
    }
  }

  return 0;
}
